import _ from "lodash";
import sdk from './sdk.js';
import {getCredentialsByType} from "./utils/sync-utils";
import {fixPath} from "./utils/encoders";
import * as consts from "./consts";
import {removeKeyFromPath} from "./utils/helpers";
import * as actionCreatorsConsts from "./action_creators_consts";
import { removeStartingAndEndingSlash } from "./utils/encoders"

export const startup = (dispatch) => {
    setTimeout(() => {
        dispatch(getRelationships());
    })
};

const thunk = (action) => (dispatch) => {
    dispatch(action);
    return action.payload;
};

const thunkMultiple = (actionArray) => (dispatch) => {
    _.each(actionArray, (action) => {
        dispatch(action);
    });

    return _.get(_.first(actionArray), 'payload');
};

const create = (type, payload, status, selfErrorHandling = false, extra = {}) => {
    return thunk({
        type,
        payload,
        status,
        selfErrorHandling,
        extra
    });
};

const createMultiple = (actionsArray) => {
    return thunkMultiple(actionsArray);
};

export const updateUrlParams = (params) => create(actionCreatorsConsts.UPDATE_URL_PARAMS, params);

export const handleUser = (r, dispatch) => {
    if (r) {
        startup(dispatch)
    }
    _lastPollerId = _lastPollerId + 1;
    dispatch(messagesPoller(r.uid, _lastPollerId));
    return r;
};


export function userInfo(token, mpCustomerToken) {
    return dispatch =>
        dispatch({
            type: actionCreatorsConsts.LOGIN_LOCAL,
            payload: sdk.userInfo({token}, mpCustomerToken).then(r => handleUser(r, dispatch))
        })
}


export const logout = () => create(actionCreatorsConsts.LOGOUT);

export const changePassword = (values) => create(actionCreatorsConsts.CHANGE_PASSWORD, sdk.changePassword(values), undefined, true); //self error handling

export const changeAWSId = (values) => create(actionCreatorsConsts.CHANGE_AWS_ID, sdk.changeAWSId(values), undefined, true); //self error handling


export const getConfig = () => create(actionCreatorsConsts.CONFIG, sdk.getConfig());

export const updateUser = (values) => create(actionCreatorsConsts.UPDATE_USER, sdk.updateUser(values), undefined, true); //self error handling

export const createDataBroker = (values) => create(actionCreatorsConsts.DATA_BROKER, sdk.createDataBroker(values), undefined, true); //self error handling

export const dataBrokers = () => create(actionCreatorsConsts.GET_DATA_BROKERS, sdk.dataBrokers());

export const getServersByType = (serverType) => create(actionCreatorsConsts.GET_SERVERS, sdk.getServers(serverType), undefined, false, {serverType: serverType});

export const azureStorageAccounts = () => create(actionCreatorsConsts.GET_AZURE_STORAGE_ACCOUNTS, sdk.azureStorageAccounts());
export const gcpProjects = () => create(actionCreatorsConsts.GET_GCP_PROJECTS, sdk.gcpProjects());


export function deleteDataBroker(id) {
    return dispatch =>
        dispatch({
            type: actionCreatorsConsts.DELETE_DATA_BROKER,
            payload: sdk.deleteDataBroker(id).then(() => dispatch(dataBrokers()))
        });
}

export const openDialog = (name) => create(actionCreatorsConsts.OPEN_DIALOG, name);

export const closeDialog = () => create(actionCreatorsConsts.CLOSE_DIALOG);

export const clearCloseSearchWidget = () => create(actionCreatorsConsts.CLEAR_CLOSE_SEARCH_WIDGET);

export const closeSearchWidget = () => create(actionCreatorsConsts.CLOSE_SEARCH_WIDGET);

export const pushUiState = (state) => create(actionCreatorsConsts.PUSH_UI_STATE, state);

export const popUiState = () => create(actionCreatorsConsts.POP_UI_STATE);

export const visitedTrialPage = () => create(actionCreatorsConsts.VISITED_TRIAL_PAGE);

export const clearUrlParamsIfChanged = (changedParam, newParamVal, objKey, without, forceClearDueToCredentialsChange) => create(actionCreatorsConsts.CLEAR_URL_PARAMS, {changedParam, newParamVal, objKey, without, forceClearDueToCredentialsChange});

export const bucketsSelectStep = () => create(actionCreatorsConsts.BUCKETS_SELECT_STEP);
export const bucketsEncryptionStep = () => create(actionCreatorsConsts.BUCKETS_ENCRYPTION_STEP);
export const scrollToTop = () => create(actionCreatorsConsts.SCROLL_TO_TOP);
export const stopScrollToTop = () => create(actionCreatorsConsts.STOP_SCROLL_TO_TOP);

export const azureSelectStep = () => create(actionCreatorsConsts.AZURE_SELECT_STEP);
export const azureAdditionalStorageSettingsStep = () => create(actionCreatorsConsts.AZURE_ADDITIONAL_SETTINGS_STEP);

export const serverCredentialsStep = (serverType) => create(actionCreatorsConsts.SERVER_CREDENTIALS_STEP, {serverType: _.camelCase(serverType.toLowerCase())});
export const serverIPStep = (serverType) => create(actionCreatorsConsts.SERVER_IP_STEP, {serverType: _.camelCase(serverType.toLowerCase())});

export const showExports = (dataBrokerId, fileServer) => {
    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.SHOW_EXPORTS,
            extra: {host: fileServer, dataBrokerId},
            payload: sdk.showExports(dataBrokerId, fileServer)
        })
    }
};

export const showEFSExports = (dataBrokerId, fileServer) => {
    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.SHOW_EXPORTS,
            extra: {host: fileServer, dataBrokerId},
            payload: sdk.showEFSExports(dataBrokerId, fileServer)
        })
    }
};

export const listFolders = (dataBrokerId, fileServer, path, exportPath, nfsVersion) => {
    const finalPath = removeKeyFromPath(path, exportPath);

    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.LIST_FOLDERS,
            extra: {fileServer, path},
            payload: sdk.listFolder(dataBrokerId, fileServer, finalPath, exportPath, nfsVersion)
        });
    }
};

export const showShares = (dataBrokerId, host, cifsVersion, isTargetOfSameSrc) => {
    const cifsCredentials = getCredentialsByType("cifs", host, isTargetOfSameSrc);
    if (!cifsCredentials) {
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_SHARES,
                status: actionCreatorsConsts.STATUS_ERROR,
                extra: {host, dataBrokerId},
                payload: {
                    data: {message: "Unable to get shares, no credentials defined"}
                }
            })
        }
    }
    else {
        const {username, password, domain} = cifsCredentials;
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_SHARES,
                extra: {host, dataBrokerId, username, password, domain, cifsVersion},
                payload: sdk.showShares(dataBrokerId, host, username, password, domain, cifsVersion)
            })
        }
    }
};

export const listShareFolders = (dataBrokerId, host, path, share, cifsVersion, isTargetOfSameSrc) => {
    const cifsCredentials = getCredentialsByType("cifs", host, isTargetOfSameSrc);
    //remove the share part from the path
    const fixedShare = fixPath(share);
    const fixedPath = removeKeyFromPath(fixPath(path), fixedShare);
    console.log(`%c listShareFolders fixedShare @ ${fixedShare} fixedPath @ (${fixedPath})`, "font-weight: bold;");
    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.LIST_SHARE_FOLDERS,
            extra: {host, path}, //put the original path since we need the starting slash in the drilldown
            payload: sdk.listShareFolders(dataBrokerId, host, fixedShare, fixedPath, cifsCredentials.username, cifsCredentials.password, cifsCredentials.domain, cifsVersion)
        });
    }
};

export const showSGBuckets = (dataBrokerId, host, isTargetOfSameSrc) => {
    const sgCredentials = getCredentialsByType("objectStorage",host, isTargetOfSameSrc);
    const {port, accessKey, secretKey} = sgCredentials;
    if (!sgCredentials) {
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_SG_BUCKETS,
                status: actionCreatorsConsts.STATUS_ERROR,
                payload: {
                    data: {message: "Unable to get StorageGRID buckets, no credentials defined"}
                },
                extra: {host: host }
            })
        }
    }
    else {
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_SG_BUCKETS,
                extra: {host: host, dataBrokerId, port, accessKey, secretKey},
                payload: sdk.fetchSGBuckets(dataBrokerId, host, sgCredentials)
            })
        }
    }
};

export const listSGBucketFolders = (dataBrokerId, host, bucket, path, isTargetOfSameSrc) => {
    const sgCredentials = getCredentialsByType("objectStorage",host, isTargetOfSameSrc);
    //remove the share part from the path
    const fixedPath = removeKeyFromPath(path, bucket);
    console.log(`%c listSGBucketFolders bucket @ ${bucket} fixedPath @ (${fixedPath})`, "font-weight: bold;");
    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.LIST_SG_BUCKETS_FOLDERS,
            extra: {host, bucket, path}, //keep the original path since we need it in the drill down
            payload: sdk.listSGBucketFolders(dataBrokerId, host, bucket, fixedPath, sgCredentials.accessKey, sgCredentials.secretKey, sgCredentials.port)
        });
    }
};

export const showGCPBuckets = (dataBrokerId, projectId, isTargetOfSameSrc) => {
    const gcpCredentials = getCredentialsByType("gcp",projectId, isTargetOfSameSrc);
    const {clientEmail, privateKey} = gcpCredentials;
    if (!gcpCredentials) {
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_GCP_BUCKETS,
                status: actionCreatorsConsts.STATUS_ERROR,
                payload: {
                    data: {message: "Unable to get GCP buckets, no credentials defined"}
                }
            })
        }
    }
    else {
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_GCP_BUCKETS,
                extra: {projectId, dataBrokerId, clientEmail, privateKey},
                payload: sdk.listGCPBuckets(dataBrokerId, projectId, clientEmail, privateKey)
            })
        }
    }
};

export const listGCPBucketFolders = (dataBrokerId, projectId, bucket, path, isTargetOfSameSrc) => {
    const gcpCredentials = getCredentialsByType("gcp",projectId, isTargetOfSameSrc);
    const {clientEmail, privateKey} = gcpCredentials;
    //remove the share part from the path
    const fixedPath = removeKeyFromPath(path, bucket);
    console.log(`%c listSGBucketFolders bucket @ ${bucket} fixedPath @ (${fixedPath})`, "font-weight: bold;");
    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.LIST_GCP_BUCKET_FOLDERS,
            extra: {projectId, bucket, path}, //keep the original path since we need it in the drill down
            payload: sdk.listGCPBucketFolders(dataBrokerId, projectId, bucket, fixedPath, clientEmail, privateKey)
        });
    }
};

export const clearSync = () => create(actionCreatorsConsts.CLEAR_SYNC);

export const clearWizardCache = (cifs, sgws, s3, azure, gcp) => create(actionCreatorsConsts.CLEAR_WIZARD_CACHE, {cifs, sgws, s3, azure, gcp});

export const addNewBucket = (bucketName, dataBrokerId) => create(actionCreatorsConsts.ADD_BUCKET_NAME, {bucketName, dataBrokerId});

export const addNewExport = (exportName, nfsServer) => create(actionCreatorsConsts.ADD_EXPORT_MANUALLY, {exportName, nfsServer});


export const createSync = (opts) => create(actionCreatorsConsts.CREATE_SYNC, sdk.createSync(opts));

export const syncNow = (relationshipId) => {
    return dispatch =>
        dispatch({
            type: actionCreatorsConsts.SYNC_NOW,
            extra: {relationshipId},
            payload: sdk.syncNow(relationshipId)
        });
};


export function abortSync(relationshipId) {
    return dispatch =>
        dispatch({
            type: actionCreatorsConsts.ABORT_SYNC,
            extra: {relationshipId},
            payload: sdk.abortSync(relationshipId)
        });
}

export const updateSyncSchedule = (relationshipId, values) => create(actionCreatorsConsts.UPDATE_SYNC_SCHEDULE, sdk.updateSyncSchedule(relationshipId, values), undefined, true); //self error handling

export const updateRelationshipSettings = (relationshipId, settings) => create(actionCreatorsConsts.UPDATE_RELATIONSHIP_SETTINGS, sdk.updateRelationship(relationshipId, settings), undefined, true); //self error handling

export const showBuckets = (dataBrokerId) => {
    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.SHOW_BUCKETS,
            extra: {dataBrokerId},
            payload: sdk.fetchBuckets(dataBrokerId)
        });
    }
};

export const listBucketFolders = (dataBrokerId, bucket, path) => {
    const fixedPath = removeKeyFromPath(path, bucket);
    return dispatch => {
        dispatch({
            type: actionCreatorsConsts.LIST_BUCKETS_FOLDERS,
            extra: {bucket, path, dataBrokerId},
            payload: sdk.listBucketFolders(dataBrokerId, bucket, fixedPath)
        });
    }
};

export const showAzureBlobContainers = (dataBrokerId, storageAccountName, isTargetOfSameSrc) => {
    const azureCredentialsObject = getCredentialsByType("azure", storageAccountName, isTargetOfSameSrc);
    if (!azureCredentialsObject) {
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_AZURE_BLOB_CONTAINERS,
                status: actionCreatorsConsts.STATUS_ERROR,
                extra: {storageAccountName, dataBrokerId},
                payload: {
                    data: {message: "Unable to get azure containers, storage account connection string is not defined"}
                }
            })
        }
    }
    else {
        const {connectionString} = azureCredentialsObject;
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_AZURE_BLOB_CONTAINERS,
                extra: {dataBrokerId, storageAccountName, connectionString},
                payload: sdk.fetchAzureBlobContainers(dataBrokerId, storageAccountName, connectionString)
            })
        }
    }
};

export const listAzureBlobContainerFolders = (dataBrokerId, storageAccountName, container, path, isTargetOfSameSrc) => {
    const azureCredentialsObject = getCredentialsByType("azure", storageAccountName, isTargetOfSameSrc);
    const fixedPath = removeKeyFromPath(path, container);
    if (!azureCredentialsObject) {
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.SHOW_AZURE_BLOB_CONTAINERS,
                status: actionCreatorsConsts.STATUS_ERROR,
                extra: {storageAccountName, dataBrokerId},
                payload: {
                    data: {message: "Unable to get azure containers, storage account conenction string is not defined"}
                }
            })
        }
    }
    else {
        const {connectionString} = azureCredentialsObject;
        return dispatch => {
            dispatch({
                type: actionCreatorsConsts.LIST_CONTAINERS_FOLDERS,
                extra: {container, path, storageAccountName, connectionString, dataBrokerId},
                payload: sdk.listContainerFolders(dataBrokerId, storageAccountName, connectionString, container, removeStartingAndEndingSlash(fixedPath))
            })
        }
    }
};



export const getRelationships = () => {
    return createMultiple([{type: actionCreatorsConsts.GET_RELATIONSHIPS, payload: sdk.getRelationships()},
        {type: actionCreatorsConsts.GET_LICENSE_SETTINGS, payload: sdk.getLicenseSettings()}]); // for license enforcement
};

export const getTimeline = () => create(actionCreatorsConsts.GET_TIMELINE, sdk.getTimeline());

export const getCost = (opts) => create(actionCreatorsConsts.GET_COST, sdk.getCost(opts));
export const getCostTimes = () => create(actionCreatorsConsts.GET_COST_TIMES, sdk.getCostTimes());

export const getLicenseSettings = () => create(actionCreatorsConsts.GET_LICENSE_SETTINGS, sdk.getLicenseSettings());
export const getBYOLLicenses = () => create(actionCreatorsConsts.GET_BYOL_LICENSES, sdk.getBYOLLicenses());
export const updateNSSCredentials = (values) => create(actionCreatorsConsts.UPDATE_NSS_CREDENTIALS, sdk.updateNSSCredentials(values), undefined, true);
export const addBYOLLicense = (values) => create(actionCreatorsConsts.ADD_BYOL_LICENSE, sdk.addBYOLLicense(values), undefined, true);

export function deleteRelationship(id) {
    return dispatch =>
        dispatch({
            type: actionCreatorsConsts.DELETE_RELATIONSHIP,
            extra: {id},
            payload: sdk.deleteRelationship(id).then(() => dispatch(getRelationships()))
        });
}

export const startWizard = (name, stepNamesByWizardType, stepsMapping) => create(actionCreatorsConsts.START_WIZARD, {
    name,
    stepNamesByWizardType,
    stepsMapping
});
export const wizardStepStarted = (name) => create(actionCreatorsConsts.STARTED_WIZARD_STEP, name);
export const wizardStepExited = (name) => create(actionCreatorsConsts.EXITED_WIZARD_STEP, name);

let lastMessage;

let _lastPollerId = 0;
function messagesPoller(uid, lastPollerId) {
    return (dispatch, getState) => {
        const nextPoll = _.once(() => {
            clearTimeout(timeout);
            if (_.get(getState(), "syncReducer.auth.succeeded") && _lastPollerId === lastPollerId) dispatch(messagesPoller(uid, lastPollerId));
        });

        const timeout = setTimeout(nextPoll, 31000);

        return sdk.messages(lastMessage)
            .then(r => {
                if (_.get(getState(), "syncReducer.auth.data.uid") === uid) handleMessages(dispatch, getState().syncReducer._pendingRequests, r);
            })
            .catch(e => {
                //don't send error message for unauthorized response on get messages, to avoid showing it in the general error panel
                if (e.status !== 401) {
                    dispatch({
                        type: actionCreatorsConsts.MESSAGES,
                        status: actionCreatorsConsts.STATUS_ERROR,
                        payload: e
                    });
                }
            })
            .then(nextPoll, nextPoll)
    }
}

const createUpdater = (type, payload) => {
    const actualPayload = _.values(payload)[0];
    payload.data = actualPayload;

    return create(type, payload, actualPayload.status === "FAILED" ? actionCreatorsConsts.STATUS_ERROR : undefined)
};

export const updateAgentStatus = (payload) => create(actionCreatorsConsts.UPDATE_AGENT_STATUS, payload, undefined); //handles errors by itself
export const updateExports = (payload) => createUpdater(actionCreatorsConsts.UPDATE_EXPORTS, payload);
export const updateShares = (payload) => createUpdater(actionCreatorsConsts.UPDATE_SHARES, payload);
export const updateFolders = (payload) => createUpdater(actionCreatorsConsts.UPDATE_FOLDERS, payload);
export const updateShareFolders = (payload) => createUpdater(actionCreatorsConsts.UPDATE_SHARE_FOLDERS, payload);
export const updateBuckets = (payload) => createUpdater(actionCreatorsConsts.UPDATE_BUCKETS, payload);
export const updateBucketsFolders = (payload) => createUpdater(actionCreatorsConsts.UPDATE_BUCKETS_FOLDERS, payload);
export const updateContainers = (payload) => createUpdater(actionCreatorsConsts.UPDATE_CONTAINERS, payload);
export const updateContainersFolders = (payload) => createUpdater(actionCreatorsConsts.UPDATE_CONTAINERS_FOLDERS, payload);
export const updateSGBuckets = (payload) => createUpdater(actionCreatorsConsts.UPDATE_SG_BUCKETS, payload);
export const updateSGBucketsFolders = (payload) => createUpdater(actionCreatorsConsts.UPDATE_SG_BUCKETS_FOLDERS, payload);
export const updateGCPBuckets = (payload) => createUpdater(actionCreatorsConsts.UPDATE_GCP_BUCKETS, payload);
export const updateGCPBucketFolders = (payload) => createUpdater(actionCreatorsConsts.UPDATE_GCP_BUCKET_FOLDERS, payload);
export const updateTimeline = (payload) => create(actionCreatorsConsts.UPDATE_TIMELINE, payload);
export const updateRelationship = (payload) => create(actionCreatorsConsts.UPDATE_RELATIONSHIP, payload);//handles errors by itself
export const updateSubscriptionStatus = (payload) => create(actionCreatorsConsts.UPDATE_SUBSCRIPTION_STATUS, payload);//handles errors by itself
export const updateDatabrokerTransferRate = (payload) => create(actionCreatorsConsts.UPDATE_DATABROKER_TRANSFER_RATE, payload);

export const toggleErrorsState = () => create(actionCreatorsConsts.TOGGLE_ERRORS_STATE);
export const removeError = (id) => create(actionCreatorsConsts.REMOVE_ERROR, id);
export const broadcastErrorsPanelSize = (rect) => create(actionCreatorsConsts.BROADCAST_ERRORS_PANEL_SIZE, rect);
export const clearContentHeight = () => create(actionCreatorsConsts.CLEAR_CONTENT_HEIGHT);

function handleMessages(dispatch, pendingRequests, messages) {
    messages.forEach(message => {
        const pendingRequest = pendingRequests[message.header.requestId];
        lastMessage = message.id;
        message.payload.requestId = message.header.requestId;
        if (pendingRequest && pendingRequest.type === actionCreatorsConsts.GET_DATA_BROKERS) {
            dispatch(updateAgentStatus(message.payload));
        } else if (message.payload[consts.PAYLOAD_DATA_BROKER_STATUS]) {
            dispatch(updateAgentStatus(message.payload));
        } else if (message.payload[consts.PAYLOAD_SHOW_EXPORTS]) { //list exports
            dispatch(updateExports(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_CIFS_SHARES]) { //list cifs share
            dispatch(updateShares(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_FOLDERS]) { //list sub folders of export
            dispatch(updateFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]) { //list sub folders of shares
            dispatch(updateShareFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_BUCKETS]) { //buckets
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateBuckets(message.payload)); //sub directories of buckets
        } else if (message.payload[consts.PAYLOAD_LIST_SUB_DIRECTORIES]) {
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateBucketsFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_CONTAINERS]) { //azure blob containers
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateContainers(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_CONTAINER_SUB_DIRECTORIES]) { //sub directories of containers
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateContainersFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_SGWS_BUCKETS]) { //sgws buckets
            dispatch(updateSGBuckets(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]) { //sub directories of sgws buckets
            dispatch(updateSGBucketsFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_GCP_BUCKETS]) { //sgws buckets
            dispatch(updateGCPBuckets(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_GCP_BUCKET_FOLDERS]) { //sub directories of sgws buckets
            dispatch(updateGCPBucketFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_TIMELINE]) {
            dispatch(updateTimeline(message.payload));
        } else if (message.payload[consts.PAYLOAD_UPDATE_RELATIONSHIP]) {
            dispatch(updateRelationship(message.payload))
        } else if (message.payload[consts.PAYLOAD_UPDATE_SUBSCRIPTION_STATUS]) {
            dispatch(updateSubscriptionStatus(message.payload))
        } else if (message.payload[consts.PAYLOAD_DATABROKER_TRANSFER_RATE]) {
            dispatch(updateDatabrokerTransferRate(message.payload))
        }
    })
}
