import React, {Component} from 'react';
import ReactDOM from 'react-dom';
import * as serviceWorker from './serviceWorker';
import TabHeader from "./components/tabs/TabHeader"
import DragSourceAndTargetStep from './components/syncWizard/DragSourceAndTargetStep';
import SelectSourceAndTargetHeader from './components/syncWizard/SelectSourceAndTargetHeader';
import Timeline from "./components/tabs/Timeline";
import LicenseSettings from "./components/tabs/licenseSettings/LicenseSettings";
import AccountSettings from "./components/tabs/AccountSettings";
import SyncSettingsEdit from "./components/tabs/syncSettings/SyncSettingsEdit";
import Cost from "./components/tabs/cost/Cost";
import {Provider, connect} from 'react-redux';
import * as actionCreatorsConsts from './action_creators_consts';
import * as actionCreators from './action_creators';
import mapStateToProps from './mapStateToProps';
import {store, history} from './store';
import sdk from "./sdk";
import {Route, Switch, withRouter, Redirect} from 'react-router-dom';
import _ from "lodash";
import {bindActionCreators} from "redux";
import {ConnectedRouter, push, replace} from "connected-react-router";
import {openDialog} from "./components/dialog/Dialog";
import ThanksForSubscribingDialog from "./components/tabs/licenseSettings/ThanksForSubscribingDialog";
import numeral from "numeral";
import Dashboard from "./components/tabs/Dashboard";
import ContentWrapper from "./components/ContentWrapper";
import WizardContent from "./components/wizard/WizardContent";
import WizardHeader from "./components/wizard/WizardHeader";
import SyncHeader from "./components/wizard/SyncHeader";
import OnPremDataBrokerInstructions from "./components/externals/OnPremDataBrokerInstructions";
import AzureDataBrokerInstructions from "./components/externals/AzureDataBrokerInstructions";
import GcpDataBrokerInstructions from "./components/externals/GcpDataBrokerInstructions";
import Auth from './auth/auth';
import {buildUrlParams} from "./utils/encoders";
import axios from 'axios'
require('./index.scss');
require("./assets/images/netapp-logo-eula.jpg");

const opts = {
    clientID: process.env.REACT_APP_AUTH_CLIENT,
    domain: process.env.REACT_APP_AUTH_DOMAIN,
    audience: process.env.REACT_APP_AUTH_AUDIENCE
};
console.log(`%c Creating auth with opts ${JSON.stringify(opts)}`, 'font-weight:bold;color:green');
window.auth = new Auth(opts);


Object.assign(numeral.locales["en"].abbreviations, {thousand: "K", million: "M", billion: "B", trillion: "T"});

if (window.navigator.userAgent.indexOf("MSIE 10") > -1 || window.navigator.userAgent.match(/Trident\//)) {
    document.getElementById('noIE').className = "show";
    document.getElementById('app').className = "hide";
} else {
    actionCreatorsConsts.routesContainer.routes = {
        push: bindActionCreators(push, store.dispatch),
        replace: bindActionCreators(replace, store.dispatch)
    };

    window.__$$expose = {
        actionCreators,
        store,
        sdk
    };

    store.dispatch(actionCreators.getConfig());

    sdk.interceptors().response.use(null, function (error) {
        if (error.response && (error.response.status === 401)) {
            const errorMessage = error.response.data ? error.response.data.message : error.response.message;
            if (_.isEqual(errorMessage, "unauthorized") || _.isEqual(errorMessage, "TokenExpiredError: jwt expired") || _.isEqual(error.response.statusText, "Unauthorized")) {
                //if we are in sso login, and there is an error other than unauthorized, we don't want to logout, to get to the oops page
                store.dispatch(actionCreators.logout());
            }
        }
        return Promise.reject(error.response);
    });


    history.listen(location => {
        window.ga && window.ga('send', 'pageview', location.pathname);
        window.fbq && window.fbq('track', 'PageView');
        window.lin && window.lin();
    });

    class RedirectorDumb extends Component {

        redirect({auth, _ui, _relationships, _licenseSettings}) {
            if (!_.get(_licenseSettings, 'succeeded')) return; // wait for license data

            // if user is subscribing
            if (_.get(auth, "data.subscribing")) {
                // show thanks dialog on top on the appropriate screen
                openDialog(<ThanksForSubscribingDialog className="confirm thanks-for-subscribing"/>);
                // do not return in order to show the appropriate screen below
            }
            // else if user needs to pay (has relationships that are not payed via subscription or license)
            else if (_ui.showTrialPage) {
                // show the trial screen with payment options
                actionCreatorsConsts.routesContainer.routes.replace({pathname: "/license", query: {trial: true}});
                return;
            }

            if (!_relationships || !_relationships.data) return; // wait for relationships data

            if (_relationships.data.length > 0) actionCreatorsConsts.routesContainer.routes.replace("/dashboard");
            else actionCreatorsConsts.routesContainer.routes.replace("/drag-source-and-target");
        }

        componentDidMount() {
            console.log(`%%%%%%%%%%%%%%%%%%%% \n ${window.location.href} \n %%%%%%%%%%%%%%%%%%%%%`);
            this.redirect(this.props);
        }

        UNSAFE_componentWillReceiveProps(nextProps) {
            console.log(`%%%%%%%%%%%%%%%%%%%% \n ${window.location.href} \n %%%%%%%%%%%%%%%%%%%%%`);
            this.redirect(nextProps);
        }

        render() {
            console.log(`================ Redirect render was called for location ${JSON.stringify(this.props.location)}`);

            return (<div></div>);
        }
    }

    const Redirector = withRouter(connect(mapStateToProps, actionCreators)(RedirectorDumb));

    class App extends Component {
        componentDidMount() {
            const urlParams = buildUrlParams(this.props._routing, this.props.match, this.props.location, window.location.href);
            this.props.updateUrlParams(urlParams);
            const {location: {query}} = urlParams;

            this.setState({attemptingSso: true});


            const loginFail = (err) => {
                console.log("SSO return error");
                console.log(err);
                if (err.needLogin) { //for any other error we don't want to login again, we want to show the oops (handled later in the render)
                    const rtQueryParam = query.rt;
                    const cameFromMarketPlace = !_.isEmpty(rtQueryParam);
                    const displayEvent = cameFromMarketPlace ? {
                        level: "message",
                        title: "Thank you for subscribing to Cloud Sync service",
                        message: "To complete the subscription, you must log in. New users should sign up."
                    } : undefined;
                    const paramsForUI = displayEvent ? {rt: rtQueryParam} : undefined;
                    window.auth.loginRedirect({}, {
                        startOnSignup: (query && query.su && query.su === "t" ? true : false),
                        service: "cloud-sync",
                        displayEvent,
                    }, paramsForUI);
                }
            };

            const loginSuccess = (authResult) => {
                console.log("SSO return success");
                console.log(authResult);
                const loginToken = `Bearer ${authResult.accessToken}`;
                const marketPlaceToken = authResult.paramsForUI.rt ? authResult.paramsForUI.rt : query.rt;
                console.log(`>>>>>>>>>>>>>>> token:\n${loginToken} token payload:\n${JSON.stringify(authResult.idTokenPayload)}\nrtFromParamsForUI: ${marketPlaceToken}`);
                window.localStorage.auth0LoginToken = loginToken;

                axios.defaults.headers.common['Authorization'] = loginToken;
                this.setState({authenticated: true, attemptingSso: false, auth: authResult.idTokenPayload});
                //Automatic login
                store.dispatch(actionCreators.userInfo(loginToken, marketPlaceToken ? marketPlaceToken : undefined));
                setTimeout(() => {
                    window.auth.refreshSso().then(loginSuccess, loginFail);
                }, authResult.expiresIn * 1000 - 10000);
            };

            window.auth.refreshSso().then(loginSuccess).catch(loginFail);
        }

        render() {
            const {auth} = this.props;
            if (!this.state || this.state.attemptingSso || !auth || !(auth.data || auth.error)) {
                return <div></div>;
            }
            else {
                return (
                    <Switch>
                        <Route exact path="/" render={(props) =>
                            <ContentWrapper Content={Redirector}
                                            contentUiState=""
                                            {...props}/>
                        }/>
                        <Route path="/dashboard" render={(props) =>
                            <ContentWrapper PrimaryHeader={TabHeader}
                                            Content={Dashboard}
                                            contentUiState="tab"
                                            {...props}/>
                        }/>
                        <Route path="/timeline" render={(props) =>
                            <ContentWrapper PrimaryHeader={TabHeader}
                                            Content={Timeline}
                                            contentUiState="tab darkula"
                                            {...props}/>
                        }/>
                        <Route path="/license" render={(props) =>
                            <ContentWrapper PrimaryHeader={TabHeader}
                                            Content={LicenseSettings}
                                            contentUiState="tab whiter"
                                            {...props}/>
                        }/>
                        <Route path="/syncSettings/:relationshipId" render={(props) =>
                            <ContentWrapper PrimaryHeader={TabHeader}
                                            Content={SyncSettingsEdit}
                                            contentUiState="tab"
                                            {...props}/>
                        }/>
                        <Route path="/accountSettings" render={(props) =>
                            <ContentWrapper PrimaryHeader={TabHeader}
                                            Content={AccountSettings}
                                            contentUiState="tab whiter"
                                            {...props}/>
                        }/>
                        <Route path="/cost" render={(props) =>
                            <ContentWrapper PrimaryHeader={TabHeader}
                                            Content={Cost}
                                            contentUiState="tab"
                                            {...props}/>
                        }/>
                        <Route exact path={`/onprem`} render={(props) =>
                            <ContentWrapper Content={OnPremDataBrokerInstructions}
                                            contentUiState="external"
                                            contentKey="externals"
                                            {...props}/>

                        }/>
                        <Route exact path={`/azure`} render={(props) =>
                            <ContentWrapper Content={AzureDataBrokerInstructions}
                                            contentUiState="external"
                                            contentKey="externals"
                                            {...props}/>

                        }/>
                        <Route exact path={`/gcp`} render={(props) =>
                            <ContentWrapper Content={GcpDataBrokerInstructions}
                                            contentUiState="external"
                                            contentKey="externals"
                                            {...props}/>

                        }/>
                        <Route path="/drag-source-and-target" render={(props) =>
                            <ContentWrapper PrimaryHeader={SyncHeader} hideMainHeader={true}
                                            Content={DragSourceAndTargetStep}
                                            ContentHeader={SelectSourceAndTargetHeader}
                                            contentUiState="wizard"
                                            {...props}/>

                        }/>

                        <Route path={`/sync/:wizardType/:firstStep`} render={(props) =>
                            <ContentWrapper PrimaryHeader={SyncHeader} hideMainHeader={true}
                                            Content={WizardContent}
                                            ContentHeader={WizardHeader}
                                            contentUiState="wizard"
                                            contentKey="sync-wizard"
                                            {...props}/>

                        }/>
                        <Redirect to="/"/>
                    </Switch>
                );
            }
        }
    }

    const withRouterApp = withRouter(connect(mapStateToProps, actionCreators)(App));

    ReactDOM.render(
        <Provider store={store}>
            <ConnectedRouter history={history}>
                <Switch>
                    <Route path="/" component={withRouterApp}/>
                </Switch>
            </ConnectedRouter>
        </Provider>,
        document.getElementById('root')
    );
}

// If you want your app to work offline and load faster, you can change
// unregister() to register() below. Note this comes with some pitfalls.
// Learn more about service workers: http://bit.ly/CRA-PWA
serviceWorker.unregister();

