import {createStore, applyMiddleware} from 'redux';
import {routerMiddleware} from 'connected-react-router';
import {createBrowserHistory} from 'history';
import thunk from 'redux-thunk'
import buildReducer from './reducers/reducer';
import {createLogger} from 'redux-logger';
import * as actions from "./action_creators_consts";
import _ from "lodash";
import RavenMiddleware from 'redux-raven-middleware';
import {uuid} from "./utils/helpers"

export const history = createBrowserHistory();
const reducer = buildReducer(history);

const predicate = (getState, action) => {
    if ((action.type === actions.UPDATE_RELATIONSHIP) && !(window.location.href.includes("dashboard") || window.location.href.includes("review"))) return false;
    if ((action.type === actions.UPDATE_AGENT_STATUS) && !(window.location.href.includes("dashboard") || window.location.href.endsWith("agents"))) return false;
    return true;
};

const logger = createLogger({predicate});

const router = routerMiddleware(history);

const promise = ({dispatch}) => {
    return next => action => {
        if(action.payload && action.payload.then && _.isFunction(action.payload.then)){
            const id = uuid();
            action.payload.then(
                    r => dispatch(Object.assign({}, action, {payload: r, id, requestId: r ? r.requestId : undefined, status: actions.STATUS_SUCCESS})),
                    e => dispatch(Object.assign({}, action, {payload: e, id, status: actions.STATUS_ERROR})));

            next(Object.assign({}, action, {id, payload: null, status: actions.STATUS_PROGRESS}));
        }
        else{
            next(action);
        }
    }
};

function configureStore(initialState) {
    const store = createStore(
        reducer,
        initialState,
        applyMiddleware(RavenMiddleware('https://32cc596b1c1345f983855e1932b33582@sentry.io/148201'), router, thunk, promise, logger)
    );

/*
    if (module.hot) {
        // Enable Webpack hot module replacement for reducers
        module.hot.accept('../reducer', () => {
            const nextRootReducer = require('../reducers').default
            store.replaceReducer(nextRootReducer)
        })
    }

*/
    return store
}

export const store = configureStore();
