import _ from "lodash";
import {
    FM_AGENT_NO_NetApp,
    DEFAULT_NFS_VERSION,
    DEFAULT_CIFS_VERSION,
    NFS_END_POINT,
    CIFS_END_POINT,
    EFS_END_POINT,
    S3_END_POINT,
    SG_END_POINT,
    IBM_END_POINT,
    AZURE_END_POINT,
    GCP_END_POINT,
    CVS_END_POINT, ANF_END_POINT,
    DATA_BROKER_FUNCTION,
    NFS_SERVER_TYPES
} from '../consts';
import {stringToTags} from "./helpers";
import {getSettingsFromSessionStorage} from "./settings-utils";
import {removeStartingAndEndingSlash, decodeAll} from "./encoders";
import DataBrokerStep from '../components/syncWizard/DataBrokerStep';
import DataBrokerFunctionStep from '../components/syncWizard/DataBrokerFunctionStep';
import ShowBucketsStep from '../components/syncWizard/ShowBucketsStep';
import ShowBucketsStepTarget from '../components/syncWizard/ShowBucketsStepTarget';
import ShowGCPBucketsStep from '../components/syncWizard/ShowGCPBucketsStep';
import ShowSGBucketsStep from '../components/syncWizard/ShowSGBucketsStep';
import NFSServerStep from '../components/syncWizard/NFSServerStep';
import CIFSServerStep from '../components/syncWizard/CIFSServerStep';
import AzureBlobStep from '../components/syncWizard/AzureBlobStep';
import SGServerStep from '../components/syncWizard/SGServerStep';
import GCPServerStep from '../components/syncWizard/GCPProjectStep';
import ShowExportsStep from '../components/syncWizard/ShowExportsStep';
import ShowSharesStep from '../components/syncWizard/ShowSharesStep';
import ShowAzureBlobContainersStep from '../components/syncWizard/ShowAzureBlobContainersStep';
import RelationshipTagsStep from '../components/syncWizard/RelationshipTagsStep';
import SyncSettingsStep from '../components/syncWizard/SyncSettingsStep';
import RelationshipStep from '../components/syncWizard/RelationshipStep';


export const END_POINT_FILE_SYSTEM = "fileSystem";
export const END_POINT_STORAGE = "storage";

export const isSourceTargetSupported = (source, target) => {
    let supportedCombinations = [
        "nfs-s3",
        "nfs-sgws",
        "nfs-gcp",
        "nfs-ibm",
        "nfs-nfs",
        "nfs-efs",
        "nfs-cvs",
        "nfs-anf",
        "nfs-azure",
        "s3-efs",
        "s3-nfs",
        "s3-cifs",
        "s3-cvs",
        "s3-anf",
        "s3-s3",
        "s3-sgws",
        "cifs-s3",
        "cifs-sgws",
        "cifs-ibm",
        "cifs-cifs",
        "cifs-azure",
        "cifs-gcp",
        "efs-s3",
        "efs-nfs",
        "azure-nfs",
        "azure-cifs",
        "azure-azure",
        "sgws-cifs",
        "sgws-nfs",
        "sgws-s3",
        "ibm-ibm",
        "sgws-sgws",
        "gcp-nfs",
        "gcp-cifs",
        "ibm-cifs",
        "ibm-nfs",
        "cvs-s3",
        "cvs-nfs",
        "anf-nfs",
        "anf-s3",
        "anf-anf",
        "efs-anf",
        "anf-azure",
        "azure-anf"
    ];
    const combination = source.type + "-" + target.type;
    return _.includes(supportedCombinations, combination);
};

export const stepNamesByWizardType = {
    "s3-nfs": [FM_AGENT_NO_NetApp, `${S3_END_POINT} Bucket`, NFS_END_POINT, "Directories", "Settings", "Review"],
    "s3-cifs": [FM_AGENT_NO_NetApp, `${S3_END_POINT} Bucket`, CIFS_END_POINT, "Shares", "Settings", "Review"],
    "s3-efs": [FM_AGENT_NO_NetApp, `${S3_END_POINT} Bucket`, EFS_END_POINT, "Directories", "Settings", "Review"],
    "s3-cvs": [FM_AGENT_NO_NetApp, `${S3_END_POINT} Bucket`, CVS_END_POINT, "Directories", "Settings", "Review"],
    "s3-anf": [FM_AGENT_NO_NetApp, `${S3_END_POINT} Bucket`, ANF_END_POINT, "Directories", "Settings", "Review"],
    "s3-sgws": [FM_AGENT_NO_NetApp, `${S3_END_POINT} Bucket`, SG_END_POINT, "SG Bucket", "Settings", "Review"],
    "s3-s3": [FM_AGENT_NO_NetApp,`${S3_END_POINT} Bucket`, `Target ${S3_END_POINT} Bucket`, "Settings", "Relationship Tags", "Review"],
    "nfs-nfs": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${NFS_END_POINT}`, "Target Directories", "Settings", "Review"],
    "snfs-snfs": [NFS_END_POINT, DATA_BROKER_FUNCTION, FM_AGENT_NO_NetApp, "Directories", `Target ${NFS_END_POINT}`, `Target ${FM_AGENT_NO_NetApp}`, "Target Directories", "Settings", "Review"],
    "nfs-s3": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `${S3_END_POINT} Bucket`, "Settings", "Relationship Tags", "Review"],
    "nfs-sgws": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", SG_END_POINT, "SG Bucket", "Settings", "Review"],
    "nfs-gcp": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", GCP_END_POINT, "GCP Bucket", "Settings", "Review"],
    "nfs-ibm": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", IBM_END_POINT, "IBM Bucket", "Settings", "Review"],
    "nfs-efs": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${EFS_END_POINT}`, "Target Directories", "Settings", "Review"],
    "nfs-cvs": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${CVS_END_POINT}`, "Target Directories", "Settings", "Review"],
    "nfs-anf": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${ANF_END_POINT}`, "Target Directories", "Settings", "Review"],
    "nfs-azure": [NFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", AZURE_END_POINT, "Azure Blob Containers", "Settings", "Relationship Tags", "Review"],
    "efs-s3": [EFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `${S3_END_POINT} Bucket`, "Settings", "Relationship Tags", "Review"],
    "efs-nfs": [EFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${NFS_END_POINT}`, "Target Directories", "Settings", "Review"],
    "cifs-cifs": [CIFS_END_POINT, FM_AGENT_NO_NetApp, "Shares", `Target ${CIFS_END_POINT}`, "Target Shares", "Settings", "Review"],
    "cifs-s3": [CIFS_END_POINT, FM_AGENT_NO_NetApp, "Shares", `${S3_END_POINT} Bucket`, "Settings", "Review"],
    "cifs-sgws": [CIFS_END_POINT, FM_AGENT_NO_NetApp, "Shares", SG_END_POINT, "SG Bucket", "Settings", "Review"],
    "cifs-ibm": [CIFS_END_POINT, FM_AGENT_NO_NetApp, "Shares", IBM_END_POINT, "IBM Bucket", "Settings", "Review"],
    "cifs-azure": [CIFS_END_POINT, FM_AGENT_NO_NetApp, "Shares", AZURE_END_POINT, "Azure Blob Containers", "Settings", "Relationship Tags", "Review"],
    "cifs-gcp": [CIFS_END_POINT, FM_AGENT_NO_NetApp, "Shares", GCP_END_POINT, "GCP Bucket", "Settings", "Review"],
    "azure-nfs": [FM_AGENT_NO_NetApp, AZURE_END_POINT, "Azure Blob Containers", NFS_END_POINT, "Directories", "Settings", "Review"],
    "azure-cifs": [FM_AGENT_NO_NetApp, AZURE_END_POINT, "Azure Blob Containers", CIFS_END_POINT, "Shares", "Settings", "Review"],
    "azure-azure": [FM_AGENT_NO_NetApp, AZURE_END_POINT, "Azure Blob Containers", `Target ${AZURE_END_POINT}`, "Target Azure Blob Containers", "Settings", "Relationship Tags", "Review"],
    "gcp-nfs": [GCP_END_POINT, FM_AGENT_NO_NetApp, "GCP Bucket", NFS_END_POINT, "Directories", "Settings", "Review"],
    "gcp-cifs": [GCP_END_POINT, FM_AGENT_NO_NetApp, "GCP Bucket",  CIFS_END_POINT, "Shares", "Settings", "Review"],
    "sgws-nfs": [SG_END_POINT, FM_AGENT_NO_NetApp, "SG Bucket", NFS_END_POINT, "Directories", "Settings", "Review"],
    "sgws-cifs": [SG_END_POINT, FM_AGENT_NO_NetApp, "SG Bucket", CIFS_END_POINT, "Shares", "Settings", "Review"],
    "sgws-s3": [SG_END_POINT, FM_AGENT_NO_NetApp, "SG Bucket", `${S3_END_POINT} Bucket`, "Settings", "Relationship Tags", "Review"],
    "sgws-sgws": [SG_END_POINT, FM_AGENT_NO_NetApp, "SG Bucket", `Target ${SG_END_POINT}`, "Target SG Bucket", "Settings", "Review"],
    "ibm-ibm": [IBM_END_POINT, FM_AGENT_NO_NetApp, "IBM Bucket", `Target ${IBM_END_POINT}`, "Target IBM Bucket", "Settings", "Review"],
    "ibm-nfs": [IBM_END_POINT, FM_AGENT_NO_NetApp, "IBM Bucket", NFS_END_POINT, "Directories", "Settings", "Review"],
    "ibm-cifs": [IBM_END_POINT, FM_AGENT_NO_NetApp, "IBM Bucket", CIFS_END_POINT, "Shares", "Settings", "Review"],
    "cvs-s3": [CVS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `${S3_END_POINT} Bucket`, "Settings", "Relationship Tags", "Review"],
    "cvs-nfs": [CVS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${NFS_END_POINT}`, "Target Directories", "Settings", "Review"],
    "anf-nfs": [ANF_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${NFS_END_POINT}`, "Target Directories", "Settings", "Review"],
    "anf-s3": [ANF_END_POINT, FM_AGENT_NO_NetApp, "Directories", `${S3_END_POINT} Bucket`, "Settings", "Relationship Tags", "Review"],
    "anf-anf": [ANF_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${ANF_END_POINT}`, "Target Directories", "Settings", "Review"],
    "efs-anf": [EFS_END_POINT, FM_AGENT_NO_NetApp, "Directories", `Target ${ANF_END_POINT}`, "Target Directories", "Settings", "Review"],
    "anf-azure": [ANF_END_POINT, FM_AGENT_NO_NetApp, "Directories", AZURE_END_POINT, "Azure Blob Containers", "Settings", "Relationship Tags", "Review"],
    "azure-anf": [FM_AGENT_NO_NetApp, AZURE_END_POINT, "Azure Blob Containers", ANF_END_POINT, "Directories", "Settings", "Review"]
};

export const stepNameToPath = {
    [FM_AGENT_NO_NetApp]: {path: "agents"},
    "Settings": {path: "settings"},
    [`Target ${FM_AGENT_NO_NetApp}`]: {path: "targetagents", isTargetStepFromSameSourceType: true},
    [DATA_BROKER_FUNCTION]: {path: "brokerfunction"},
    "AWS S3 Bucket": {path: "buckets"},
    "Target AWS S3 Bucket": {path: "targetbuckets", isTargetStepFromSameSourceType: true},
    "GCP Bucket": {path: "gcpbuckets"},
    "SG Bucket": {path: "sgbuckets"},
    [`Target ${SG_END_POINT}`]: {path: "targetsgws", isTargetStepFromSameSourceType: true},
    "IBM Bucket": {path: "sgbuckets"},
    [`Target ${IBM_END_POINT}`]: {path: "targetsgws", isTargetStepFromSameSourceType: true},
    "Target IBM Bucket": {path: "targetsgbuckets", isTargetStepFromSameSourceType: true},
    "Target SG Bucket": {path: "targetsgbuckets", isTargetStepFromSameSourceType: true},
    [NFS_END_POINT]: {path: "nfs"},
    [EFS_END_POINT]: {path: "efs"},
    [CVS_END_POINT]: {path: "cvs"},
    [ANF_END_POINT]: {path: "anf"},
    [`Target ${CVS_END_POINT}`]: {path: "targetcvs", isTargetStepFromSameSourceType: true},
    [`Target ${ANF_END_POINT}`]: {path: "targetanf", isTargetStepFromSameSourceType: true},
    [`Target ${NFS_END_POINT}`]: {path: "targetnfs", isTargetStepFromSameSourceType: true},
    [`Target ${EFS_END_POINT}`]: {path: "targetefs", isTargetStepFromSameSourceType: true},
    [`Target ${CIFS_END_POINT}`]: {path: "targetcifs", isTargetStepFromSameSourceType: true},
    [`Target ${AZURE_END_POINT}`]: {path: "targetazure", isTargetStepFromSameSourceType: true},
    [CIFS_END_POINT]: {path: "cifs"},
    [SG_END_POINT]: {path: "sgws"},
    [GCP_END_POINT]: {path: "gcp"},
    [IBM_END_POINT]: {path: "sgws"},
    "Directories": {path: "exports"},
    "Target Directories": {path: "targetexports", isTargetStepFromSameSourceType: true},
    "Target Shares": {path: "targetshares", isTargetStepFromSameSourceType: true},
    "Target Azure Blob Containers": {path: "targetblobcontainers", isTargetStepFromSameSourceType: true},
    "Shares": {path: "shares"},
    [AZURE_END_POINT]: {path: "azure"},
    "Azure Blob Containers": {path: "blobcontainers"},
    "Relationship Tags": {path: "tags"},
    "Review": {path: "review"}
};

export const matchTypeToSourceStep = {
    "nfs-nfs": [NFS_END_POINT, "Directories"],
    "snfs-snfs": [NFS_END_POINT, "Directories"],
    "nfs-s3": [NFS_END_POINT, "Directories"],
    "nfs-sgws": [NFS_END_POINT, "Directories"],
    "nfs-gcp": [NFS_END_POINT, "Directories"],
    "nfs-ibm": [NFS_END_POINT, "Directories"],
    "nfs-efs": [NFS_END_POINT, "Directories"],
    "nfs-cvs": [NFS_END_POINT, "Directories"],
    "nfs-anf": [NFS_END_POINT, "Directories"],
    "nfs-azure": [NFS_END_POINT, "Directories"],
    "s3-nfs": ["AWS S3 Bucket"],
    "s3-cifs": ["AWS S3 Bucket"],
    "s3-efs": ["AWS S3 Bucket"],
    "s3-cvs": ["AWS S3 Bucket"],
    "s3-anf": ["AWS S3 Bucket"],
    "s3-s3": ["AWS S3 Bucket"],
    "s3-sgws": ["AWS S3 Bucket"],
    "cifs-s3": [CIFS_END_POINT, "Shares"],
    "cifs-sgws": [CIFS_END_POINT, "Shares"],
    "cifs-ibm": [CIFS_END_POINT, "Shares"],
    "cifs-cifs": [CIFS_END_POINT, "Shares"],
    "cifs-azure": [CIFS_END_POINT, "Shares"],
    "cifs-gcp": [CIFS_END_POINT, "Shares"],
    "efs-s3": [EFS_END_POINT, "Directories"],
    "efs-nfs": [EFS_END_POINT, "Directories"],
    "azure-nfs": [AZURE_END_POINT, "Azure Blob Containers"],
    "azure-cifs": [AZURE_END_POINT, "Azure Blob Containers"],
    "azure-azure": [AZURE_END_POINT, "Azure Blob Containers"],
    "gcp-nfs": [GCP_END_POINT, "GCP Bucket"],
    "gcp-cifs": [GCP_END_POINT, "GCP Bucket"],
    "sgws-sgws": [SG_END_POINT, "SG Bucket"],
    "sgws-nfs": [SG_END_POINT, "SG Bucket"],
    "sgws-cifs": [SG_END_POINT, "SG Bucket"],
    "sgws-s3": [SG_END_POINT, "SG Bucket"],
    "ibm-ibm": [IBM_END_POINT, "IBM Bucket"],
    "ibm-nfs": [IBM_END_POINT, "IBM Bucket"],
    "ibm-cifs": [IBM_END_POINT, "IBM Bucket"],
    "cvs-s3": [CVS_END_POINT, "Directories"],
    "cvs-nfs": [CVS_END_POINT, "Directories"],
    "anf-nfs": [ANF_END_POINT, "Directories"],
    "anf-s3": [ANF_END_POINT, "Directories"],
    "anf-anf": [ANF_END_POINT, "Directories"],
    "efs-anf": [EFS_END_POINT, "Directories"],
    "anf-azure": [ANF_END_POINT, "Directories"],
    "azure-anf": [AZURE_END_POINT, "Azure Blob Containers"]
};

export const sourceAndTargetDisplayName = {
    "s3": "AWS S3 Bucket",
    "sgws": SG_END_POINT,
    "gcp": GCP_END_POINT,
    "ibm": IBM_END_POINT,
    "nfs": NFS_END_POINT,
    "snfs": NFS_END_POINT,
    "cifs": CIFS_END_POINT,
    "efs": EFS_END_POINT,
    "azure": AZURE_END_POINT,
    "cvs": CVS_END_POINT,
    "anf": ANF_END_POINT
};

const endPointTypes = {
    "nfs": END_POINT_FILE_SYSTEM,
    "snfs": END_POINT_FILE_SYSTEM,
    "cifs": END_POINT_FILE_SYSTEM,
    "efs": END_POINT_FILE_SYSTEM,
    "s3": END_POINT_STORAGE,
    "sgws": END_POINT_STORAGE,
    "gcp": END_POINT_STORAGE,
    "ibm": END_POINT_STORAGE,
    "azure": END_POINT_STORAGE,
    "cvs": END_POINT_FILE_SYSTEM,
    "anf": END_POINT_FILE_SYSTEM
};

export const getNextStepComponent = (nextStep) => {
    switch (nextStep) {
        case "agents":
        case "targetagents":
            return DataBrokerStep;
        case "brokerfunction":
            return DataBrokerFunctionStep;
        case "buckets":
            return ShowBucketsStep;
        case "targetbuckets":
            return ShowBucketsStepTarget;
        case "gcpbuckets":
            return ShowGCPBucketsStep;
        case "sgbuckets":
        case "targetsgbuckets":
            return ShowSGBucketsStep;
        case "nfs":
        case "efs":
        case "cvs":
        case "anf":
        case "targetcvs":
        case "targetanf":
        case "targetnfs":
        case "targetefs":
            return NFSServerStep;
        case "targetcifs":
        case "cifs":
            return CIFSServerStep;
        case "azure":
        case "targetazure":
            return AzureBlobStep;
        case "sgws":
        case "targetsgws":
            return SGServerStep;
        case "gcp":
            return GCPServerStep;
        case "targetexports":
        case "exports":
            return ShowExportsStep;
        case "shares":
        case "targetshares":
            return ShowSharesStep;
        case "blobcontainers":
        case "targetblobcontainers":
            return ShowAzureBlobContainersStep;
        case "tags":
            return RelationshipTagsStep;
        case "settings":
            return SyncSettingsStep;
        case "review":
            return RelationshipStep;
        default: {
            throw new Error(`Unknown step ${nextStep}`);
        }
    }
};

const getProtocolSecureAgnostic = (protocol) => {
    return {source: protocol.source === 'snfs' ? 'nfs' : protocol.source, target: protocol.target === 'snfs' ? 'nfs' : protocol.target};
};

// This function should be used in NFS related steps (showExports, NFSServerStep)
// examples: for nfs-s3 return nfs, for efs-nfs return according to isTargetOfSameSourceType
export const getNfsServerTypeFromWizardType = (wizardType, isTargetOfSameSourceType) => {
    const protocol = getProtocolSecureAgnostic(protocolByWizardType(wizardType));
    if (_.includes(NFS_SERVER_TYPES, protocol.target) && _.includes(NFS_SERVER_TYPES, protocol.source)) {
        return isTargetOfSameSourceType ? protocol.target : protocol.source;
    }
    if (_.includes(NFS_SERVER_TYPES, protocol.target)) return protocol.target;
    if (_.includes(NFS_SERVER_TYPES, protocol.source)) return protocol.source;
    else throw new Error(`wizardType ${wizardType} does not include any of NFS_SERVER_TYPES`);
};

export const calculateNextStepObj = (wizard, currentStep) => {
    const currentStepIndex = wizard.stepNames.indexOf(currentStep);
    const nextStepName = wizard.stepNames[currentStepIndex + 1];
    return wizard.stepsMapping[nextStepName];
};

export const getObjectStorageTargetTypeFromWizardType = (wizardType) => {
    const protocol = protocolByWizardType(wizardType);
    switch (protocol.target) {
        case "azure":
            return "Azure Blob Container";
        case "s3":
            return "S3 bucket";
        default:
            throw new Error(`Object storage of type ${protocol.target} is not defined for getObjectStorageTargetTypeFromWizardType`);
    }
};

export const protocolByWizardType = (type) => {
    const split = type.split("-");
    return {source: split[0], target: split[1]};
};

const getVersion = (version, defaultVersion) => {
    return version || defaultVersion;
};

export const appendItemToLocalStorage = (type, key, data, isTargetOfSameSrc) => {
    const formattedType = isTargetOfSameSrc ? `${type}-trg` : type;
    const keyCamelCase = _.camelCase(formattedType);
    let localCred = window.localStorage.getItem(keyCamelCase);
    localCred = (localCred === null) ? {} : JSON.parse(localCred);
    localCred[key] = data;
    window.localStorage.setItem(keyCamelCase, JSON.stringify(localCred));
};

export const getS3EncryptionKey = (path) => {
    return getCredentialsByType("s3", path.split('/')[0]);
};

export const getCredentialsByType = (type, key, isTargetOfSameSrc, remove) => {
    const formattedType = isTargetOfSameSrc ? `${type}Trg` : type;
    const credObject = JSON.parse(window.localStorage.getItem(_.camelCase(formattedType)));
    const credObjectByKey = credObject ? credObject[key] : null;
    if (remove) {
        _.each(remove, keyToRemove => delete credObjectByKey[keyToRemove]);
    }
    return credObjectByKey;
};

export const removeCredentialsByType = (type) => {
    window.localStorage.removeItem(type);
    window.localStorage.removeItem(_.camelCase(`${type}-trg`));
};

export const fileServerByRelationshipType = (type, sourceIsStorage) => {
    const protocols = protocolByWizardType(type);
    const fileServer = sourceIsStorage ? protocols.target.toUpperCase() : protocols.source.toUpperCase();
    return fileServer === 'SNFS' ? 'NFS' : fileServer;
};

export const getEndPointType = (relationshipType, sourceOrTarget) => {
    const protocols = protocolByWizardType(relationshipType);
    const endPointType = endPointTypes[protocols[sourceOrTarget]];
    if (!endPointType) throw new Error(`${sourceOrTarget} end point from the relationship type ${relationshipType} and protorocols ${protocols}, is not defined in endPointTypes`);
    return endPointType;
};

export const getSourceAndTargetInfo = ({_wizard: {
                                               selectedWizardParams: {
                                                   selectedNFS, selectedNFSTarget,
                                                   selectedCIFS, selectedCIFSTarget,
                                                   selectedSGBucketPath, selectedSGBucketPathTarget, selectedBucketPath, selectedBucketPathTarget, selectedGCPBucketPath,
                                                   selectedExport, selectedExportTarget,
                                                   selectedShare, selectedShareTarget,
                                                   selectedStorageAccountName, selectedStorageAccountNameTarget,
                                                   selectedAzureBlobContainerPath, selectedAzureBlobContainerPathTarget
                                               }, wizardType
                                           }
                                       }) => {
    const protocols = protocolByWizardType(wizardType);
    const getBucketPaths = (fullBucketPath) => {
        const [bucketName, ...restOfBucketPath] = removeStartingAndEndingSlash(fullBucketPath || "").split("/");
        const bucketPath = restOfBucketPath ? restOfBucketPath.join("/") : "";
        return {
            bucketName,
            bucketPath
        }
    };
    const infoByProtocol = (protocol, asTarget) => {
        let objectInfo = {};

        switch (protocol) {
            case "snfs" :
            case "nfs" : {
                objectInfo.title = NFS_END_POINT;
                objectInfo.description = asTarget && selectedNFSTarget ? selectedNFSTarget : selectedNFS;
                objectInfo.subtitle = "Directory";
                objectInfo.subtitleDescription = asTarget && selectedExportTarget ? selectedExportTarget : selectedExport;
                break;
            }
            case "cifs" : {
                objectInfo.title = CIFS_END_POINT;
                objectInfo.description = asTarget && selectedCIFSTarget ? selectedCIFSTarget : selectedCIFS;
                objectInfo.subtitle = "Share";
                objectInfo.subtitleDescription = asTarget && selectedShareTarget ? selectedShareTarget : selectedShare;
                break;
            }
            case "efs": {
                objectInfo.title = EFS_END_POINT;
                objectInfo.description = asTarget && selectedNFSTarget ? selectedNFSTarget : selectedNFS;
                objectInfo.subtitle = "Directory";
                objectInfo.subtitleDescription = asTarget && selectedExportTarget ? selectedExportTarget : selectedExport;
                break;
            }
            case "cvs": {
                objectInfo.title = CVS_END_POINT;
                objectInfo.description = asTarget && selectedNFSTarget ? selectedNFSTarget : selectedNFS;
                objectInfo.subtitle = "Directory";
                objectInfo.subtitleDescription = asTarget && selectedExportTarget ? selectedExportTarget : selectedExport;
                break;
            }
            case "anf": {
                objectInfo.title = ANF_END_POINT;
                objectInfo.description = asTarget && selectedNFSTarget ? selectedNFSTarget : selectedNFS;
                objectInfo.subtitle = "Directory";
                objectInfo.subtitleDescription = asTarget && selectedExportTarget ? selectedExportTarget : selectedExport;
                break;
            }
            case "s3": {
                const bucketPaths = asTarget && selectedBucketPathTarget ? getBucketPaths(selectedBucketPathTarget) : getBucketPaths(selectedBucketPath);
                objectInfo.title = "AWS S3 Bucket";
                objectInfo.description = bucketPaths.bucketName;
                objectInfo.subtitle = "Folder";
                objectInfo.subtitleDescription = bucketPaths.bucketPath;
                break;
            }
            case "sgws": {
                const bucketPaths = asTarget && selectedSGBucketPathTarget ? getBucketPaths(selectedSGBucketPathTarget) : getBucketPaths(selectedSGBucketPath);
                objectInfo.title = "StorageGRID Bucket";
                objectInfo.description = bucketPaths.bucketName;
                objectInfo.subtitle = "Folder";
                objectInfo.subtitleDescription = bucketPaths.bucketPath;
                break;
            }
            case "gcp": {
                const bucketPaths = getBucketPaths(selectedGCPBucketPath);
                objectInfo.title = "GCP Storage Bucket";
                objectInfo.description = bucketPaths.bucketName;
                objectInfo.subtitle = "Folder";
                objectInfo.subtitleDescription = bucketPaths.bucketPath;
                break;
            }
            case "ibm": {
                const bucketPaths = asTarget && selectedSGBucketPathTarget ? getBucketPaths(selectedSGBucketPathTarget) : getBucketPaths(selectedSGBucketPath);
                objectInfo.title = "IBM Cloud Object Storage Bucket";
                objectInfo.description = bucketPaths.bucketName;
                objectInfo.subtitle = "Folder";
                objectInfo.subtitleDescription = bucketPaths.bucketPath;
                break;
            }
            case "azure": {
                objectInfo.title = "Azure Blob Storage Account";
                objectInfo.description = asTarget && selectedStorageAccountNameTarget ? selectedStorageAccountNameTarget : selectedStorageAccountName;
                objectInfo.subtitle = "Container";
                objectInfo.subtitleDescription = asTarget && selectedAzureBlobContainerPathTarget ? selectedAzureBlobContainerPathTarget : selectedAzureBlobContainerPath;
                break;
            }
            default: {
                objectInfo.title = "Unknown Protocol";
                objectInfo.description = "Unknown Protocol";
                objectInfo.subtitle = "Unknown Protocol";
                objectInfo.subtitleDescription = "Unknown Protocol";
                break;
            }
        }
        return objectInfo;
    };
    const sourceInfo = infoByProtocol(protocols.source);
    const targetInfo = infoByProtocol(protocols.target, true);
    return {sourceInfo, targetInfo};
};


export const breakFullPathToRootAndPath = (fullPath, root) => {
    const trimedRoot = removeStartingAndEndingSlash(root);
    const trimedFullPath = removeStartingAndEndingSlash(fullPath);
    let pathWithoutRoot, rootOfPath;
    if (!_.isNil(trimedRoot)) {//exports and shares
        const rootLen = trimedRoot.length;
        rootOfPath = root;
        if (_.startsWith(trimedFullPath, trimedRoot))
            pathWithoutRoot = removeStartingAndEndingSlash(trimedFullPath.slice(rootLen));
        else pathWithoutRoot = trimedFullPath;
    }
    else {//in case of bucket, there is no root;
        const indexOfFirstSlash = trimedFullPath.indexOf("/");
        pathWithoutRoot = trimedFullPath.slice(indexOfFirstSlash + 1);
        rootOfPath = trimedFullPath.slice(0, indexOfFirstSlash)
    }

    return {pathWithoutRoot, rootOfPath};
};

export const buildSyncObject = ({
                                    _wizard: {
                                        selectedWizardParams: {
                                            selectedDataBroker, selectedDataBrokerTarget, selectedIsTargetDataBrokerInitiator, selectedNFS, selectedNFSTarget,
                                            selectedCIFS, selectedCIFSTarget,
                                            selectedSG, selectedSGTarget, selectedSGBucketPath, selectedSGBucketPathTarget,
                                            selectedBucketPath, selectedBucketPathTarget, selectedStorageClass,
                                            selectedExport, selectedExportTarget,
                                            selectedShare, selectedShareTarget,
                                            selectedRootExport, selectedRootExportTarget,
                                            cifsVersion, targetCifsVersion,
                                            selectedRootShare, selectedRootShareTarget,
                                            relationshipTags, nfsVersion, targetNfsVersion,
                                            selectedStorageAccountName, selectedStorageAccountNameTarget,
                                            selectedAzureBlobContainerPath, selectedAzureBlobContainerPathTarget,
                                            selectedGCPBucketPath, selectedGCPProjectId, selectedBlobTier
                                        }, wizardType
                                    }
                                }) => {

    const relationshipTagsArray = relationshipTags === "noTags" || !relationshipTags ? [] : stringToTags(relationshipTags);
    const selectedIsTargetDataBrokerInitiatorBool = (selectedIsTargetDataBrokerInitiator === "true");
    let syncRequestObject = {
        dataBrokerId: selectedIsTargetDataBrokerInitiatorBool ?  selectedDataBrokerTarget.id : selectedDataBroker.id,
        syncInDays: 1,
        syncInHours: 0,
        sourceCredentials: undefined,
        targetCredentials: undefined,
        settings: getSettingsFromSessionStorage()
    };

    const breakBucketPathToBucketAndPrefix = (bucketPath) => {
        const splittedBucketPath = _.split(bucketPath, '/');
        const bucket = splittedBucketPath[0];
        const prefix = _.size(splittedBucketPath) <= 1 ? "" : _.join(_.drop(splittedBucketPath), '/');
        return {bucket, prefix};

    };

    const getEncryption = (path) => {
        const encKey = getS3EncryptionKey(path);
        if (encKey) return {protocol: "kms", keyId: encKey};
        else return undefined;
    };


    function nfsKindObjBuilder(isTargetOfSameSource, isSourceOfSameTarget, origProtocol) {
        const brokenPath = isTargetOfSameSource ? breakFullPathToRootAndPath(selectedExportTarget, selectedRootExportTarget) : breakFullPathToRootAndPath(selectedExport, selectedRootExport);
        const host = isTargetOfSameSource ? selectedNFSTarget : selectedNFS;
        const version = isTargetOfSameSource ? getVersion(targetNfsVersion, DEFAULT_NFS_VERSION[origProtocol.toUpperCase()]) : getVersion(nfsVersion, DEFAULT_NFS_VERSION[origProtocol.toUpperCase()]);
        let dataBrokerId;
        if (origProtocol === "snfs") {
            if (isTargetOfSameSource) {
                dataBrokerId = !selectedIsTargetDataBrokerInitiatorBool ? {dataBrokerId: _.isUndefined(selectedDataBrokerTarget) ? '' : selectedDataBrokerTarget.id} : undefined;
            }
            else if (isSourceOfSameTarget) {
                dataBrokerId = selectedIsTargetDataBrokerInitiatorBool ? {dataBrokerId: _.isUndefined(selectedDataBroker) ? '' : selectedDataBroker.id} : undefined;
            }
        }

        return {
            host,
            export: brokenPath.rootOfPath,
            path: brokenPath.pathWithoutRoot,
            version,
            nfsType: origProtocol,
            ...dataBrokerId && dataBrokerId
        };
    }

    const getParamsByProtocol = (origProtocol, isTargetOfSameSource, isSourceOfSameTarget) => {
        let protocolObject;
        let protocol = origProtocol === "ibm" ? "sgws" : origProtocol;
        switch (origProtocol) {
            case "cvs":
            case "anf":
            case "efs":
            case "snfs":
            case "nfs": {
                protocol = "nfs";
                protocolObject = nfsKindObjBuilder(isTargetOfSameSource, isSourceOfSameTarget, origProtocol);
                break;
            }
            case "s3": {
                const brokenPath = isTargetOfSameSource ? breakBucketPathToBucketAndPrefix(selectedBucketPathTarget) :
                    breakBucketPathToBucketAndPrefix(selectedBucketPath);
                protocolObject = {
                    bucket: brokenPath.bucket,
                    prefix: brokenPath.prefix,
                    encryption: isSourceOfSameTarget ? undefined : (isTargetOfSameSource ? getEncryption(selectedBucketPathTarget) : getEncryption(selectedBucketPath)),
                    tags: !isSourceOfSameTarget && relationshipTags ? relationshipTagsArray : undefined,
                    storageClass: isSourceOfSameTarget ? undefined : selectedStorageClass
                };
                break;
            }
            case "cifs": {
                const brokenPath = isTargetOfSameSource ? breakFullPathToRootAndPath(selectedShareTarget, selectedRootShareTarget) : breakFullPathToRootAndPath(selectedShare, selectedRootShare);
                const version = isTargetOfSameSource ? getVersion(targetCifsVersion, DEFAULT_CIFS_VERSION) : getVersion(cifsVersion, DEFAULT_CIFS_VERSION);
                const host = isTargetOfSameSource ? selectedCIFSTarget : selectedCIFS;
                protocolObject = {host, share: brokenPath.rootOfPath, path: brokenPath.pathWithoutRoot, version};
                break;
            }
            case "azure": {
                const brokenPath = isTargetOfSameSource ? breakBucketPathToBucketAndPrefix(selectedAzureBlobContainerPathTarget) : breakBucketPathToBucketAndPrefix(selectedAzureBlobContainerPath);
                protocolObject = {
                    storageAccountName: isTargetOfSameSource ? selectedStorageAccountNameTarget : selectedStorageAccountName,
                    container: brokenPath.bucket,
                    prefix: brokenPath.prefix,
                    tags: relationshipTagsArray,
                    blobTier: isSourceOfSameTarget ? undefined : selectedBlobTier
                };
                break;
            }
            case "sgws":
            case "ibm": {
                const brokenPath = isTargetOfSameSource ? breakBucketPathToBucketAndPrefix(selectedSGBucketPathTarget) : breakBucketPathToBucketAndPrefix(selectedSGBucketPath);
                const theHost = isTargetOfSameSource ? selectedSGTarget : selectedSG;
                protocolObject = {
                    objectStorageType: origProtocol,
                    host: theHost,
                    port: getCredentialsByType("objectStorage", theHost, isTargetOfSameSource).port,
                    bucket: brokenPath.bucket,
                    prefix: brokenPath.prefix
                };
                break;
            }
            case "gcp": {
                const brokenPath = breakBucketPathToBucketAndPrefix(selectedGCPBucketPath);
                protocolObject = {
                    projectId: selectedGCPProjectId,
                    bucket: brokenPath.bucket,
                    prefix: brokenPath.prefix
                };
                break;
            }
            default: {
                throw new Error(`getParamsByProtocol unknown protocol ${origProtocol}`);
            }
        }
        return {protocol, [protocol]: protocolObject}

    };

    switch (wizardType) {
        case "nfs-nfs": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("nfs", true);
            break;
        }
        case "snfs-snfs": {
            syncRequestObject.source = getParamsByProtocol("snfs", false, true);
            syncRequestObject.target = getParamsByProtocol("snfs", true);
            break;
        }
        case "nfs-efs": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("efs", true);
            break;
        }
        case "nfs-cvs": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("cvs", true);
            break;
        }
        case "nfs-anf": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("anf", true);
            break;
        }
        case "nfs-s3": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("s3", false);
            break;
        }
        case "nfs-sgws": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("sgws", false);
            syncRequestObject.targetCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            break;
        }
        case "nfs-gcp": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("gcp", false);
            syncRequestObject.targetCredentials = {gcp: getCredentialsByType("gcp", selectedGCPProjectId, false)};
            break;
        }
        case "nfs-ibm": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("ibm", false);
            syncRequestObject.targetCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            break;
        }
        case "nfs-azure": {
            syncRequestObject.source = getParamsByProtocol("nfs", false);
            syncRequestObject.target = getParamsByProtocol("azure", false);
            const {connectionString} = getCredentialsByType("azure", selectedStorageAccountName, false);
            syncRequestObject.targetCredentials = {azure: {storageAccountConnectionString: connectionString}};
            break;
        }
        case "efs-nfs": {
            syncRequestObject.source = getParamsByProtocol("efs", false);
            syncRequestObject.target = getParamsByProtocol("nfs", true);
            break;
        }
        case "efs-s3": {
            syncRequestObject.source = getParamsByProtocol("efs", false);
            syncRequestObject.target = getParamsByProtocol("s3", false);
            break;
        }
        case "cvs-s3": {
            syncRequestObject.source = getParamsByProtocol("cvs", false);
            syncRequestObject.target = getParamsByProtocol("s3", false);
            break;
        }
        case "cvs-nfs": {
            syncRequestObject.source = getParamsByProtocol("cvs", false);
            syncRequestObject.target = getParamsByProtocol("nfs", true);
            break;
        }
        case "anf-nfs": {
            syncRequestObject.source = getParamsByProtocol("anf", false);
            syncRequestObject.target = getParamsByProtocol("nfs", true);
            break;
        }
        case "anf-s3": {
            syncRequestObject.source = getParamsByProtocol("anf", false);
            syncRequestObject.target = getParamsByProtocol("s3", false);
            break;
        }
        case "cifs-cifs": {
            syncRequestObject.source = getParamsByProtocol("cifs", false);
            const credObj = getCredentialsByType("cifs", selectedCIFS);
            const aclServer = credObj.useACL ? credObj.windowsIP : undefined;
            if (aclServer)
                syncRequestObject.source.cifs.aclServer = aclServer;
            syncRequestObject.sourceCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            syncRequestObject.target = getParamsByProtocol("cifs", true);
            syncRequestObject.targetCredentials = {cifs: getCredentialsByType("cifs", selectedCIFSTarget, true, ["serverName", "version", "useACL", "windowsIP"])};
            break;
        }
        case "cifs-s3": {
            syncRequestObject.source = getParamsByProtocol("cifs", false);
            syncRequestObject.sourceCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            syncRequestObject.target = getParamsByProtocol("s3", false);
            break;
        }
        case "cifs-sgws": {
            syncRequestObject.source = getParamsByProtocol("cifs", false);
            syncRequestObject.sourceCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS,false, ["serverName", "version", "useACL", "windowsIP"])};
            syncRequestObject.target = getParamsByProtocol("sgws", false);
            syncRequestObject.targetCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            break;
        }
        case "cifs-ibm": {
            syncRequestObject.source = getParamsByProtocol("cifs", false);
            syncRequestObject.sourceCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            syncRequestObject.target = getParamsByProtocol("ibm", false);
            syncRequestObject.targetCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            break;
        }
        case "cifs-gcp": {
            syncRequestObject.source = getParamsByProtocol("cifs", false);
            syncRequestObject.sourceCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            syncRequestObject.target = getParamsByProtocol("gcp", false);
            syncRequestObject.targetCredentials = {gcp: getCredentialsByType("gcp", selectedGCPProjectId, false)};
            break;
        }
        case "s3-efs": {
            syncRequestObject.source = getParamsByProtocol("s3", false);
            syncRequestObject.target = getParamsByProtocol("efs", false);
            break;
        }
        case "s3-nfs": {
            syncRequestObject.source = getParamsByProtocol("s3", false);
            syncRequestObject.target = getParamsByProtocol("nfs", false);
            break;
        }
        case "s3-cvs": {
            syncRequestObject.source = getParamsByProtocol("s3", false);
            syncRequestObject.target = getParamsByProtocol("cvs", false);
            break;
        }
        case "s3-anf": {
            syncRequestObject.source = getParamsByProtocol("s3", false);
            syncRequestObject.target = getParamsByProtocol("anf", false);
            break;
        }
        case "s3-cifs": {
            syncRequestObject.source = getParamsByProtocol("s3", false);
            syncRequestObject.target = getParamsByProtocol("cifs", false);
            syncRequestObject.targetCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            break;
        }
        case "s3-s3": {
            syncRequestObject.source = getParamsByProtocol("s3", false, true); //say it is source of same target to avoid putting tags that were defined in the target
            syncRequestObject.target = getParamsByProtocol("s3", true);
            break;
        }
        case "s3-sgws": {
            syncRequestObject.source = getParamsByProtocol("s3", false);
            syncRequestObject.target = getParamsByProtocol("sgws", false);
            syncRequestObject.targetCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            break;
        }
        case "cifs-azure": {
            syncRequestObject.source = getParamsByProtocol("cifs", false);
            syncRequestObject.sourceCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            syncRequestObject.target = getParamsByProtocol("azure", false);
            const {connectionString} = getCredentialsByType("azure", selectedStorageAccountName, false);
            syncRequestObject.targetCredentials = {azure: {storageAccountConnectionString: connectionString}};
            break;
        }
        case "azure-cifs": {
            syncRequestObject.source = getParamsByProtocol("azure", false);
            const {connectionString} = getCredentialsByType("azure", selectedStorageAccountName, false);
            syncRequestObject.sourceCredentials = {azure: {storageAccountConnectionString: connectionString}};
            syncRequestObject.target = getParamsByProtocol("cifs", false);
            syncRequestObject.targetCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            break;
        }
        case "azure-nfs": {
            syncRequestObject.source = getParamsByProtocol("azure", false);
            const {connectionString} = getCredentialsByType("azure", selectedStorageAccountName, false);
            syncRequestObject.sourceCredentials = {azure: {storageAccountConnectionString: connectionString}};
            syncRequestObject.target = getParamsByProtocol("nfs", false);
            break;
        }
        case "azure-azure": {
            syncRequestObject.source = getParamsByProtocol("azure", false);
            const {connectionString} = getCredentialsByType("azure", selectedStorageAccountName, false);
            syncRequestObject.sourceCredentials = {azure: {storageAccountConnectionString: connectionString}};
            syncRequestObject.target = getParamsByProtocol("azure", true);
            const {connectionString: connectionStringTarget} = getCredentialsByType("azure", selectedStorageAccountNameTarget, true);
            syncRequestObject.targetCredentials = {azure: {storageAccountConnectionString: connectionStringTarget}};
            break;
        }
        case "sgws-cifs": {
            syncRequestObject.source = getParamsByProtocol("sgws", false);
            syncRequestObject.sourceCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            syncRequestObject.target = getParamsByProtocol("cifs", false);
            syncRequestObject.targetCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            break;
        }
        case "ibm-cifs": {
            syncRequestObject.source = getParamsByProtocol("ibm", false);
            syncRequestObject.sourceCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            syncRequestObject.target = getParamsByProtocol("cifs", false);
            syncRequestObject.targetCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            break;
        }
        case "sgws-nfs": {
            syncRequestObject.target = getParamsByProtocol("nfs", false);
            syncRequestObject.source = getParamsByProtocol("sgws", false);
            syncRequestObject.sourceCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            break;
        }
        case "sgws-s3": {
            syncRequestObject.source = getParamsByProtocol("sgws", false);
            syncRequestObject.sourceCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            syncRequestObject.target = getParamsByProtocol("s3", false);
            break;
        }

        case "sgws-sgws": {
            syncRequestObject.source = getParamsByProtocol("sgws", false);
            syncRequestObject.sourceCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            syncRequestObject.target = getParamsByProtocol("sgws", true);
            syncRequestObject.targetCredentials = {sgws: getCredentialsByType("objectStorage", selectedSGTarget, true, ["serverName", "port"])};
            break;
        }

        case "gcp-nfs": {
            syncRequestObject.target = getParamsByProtocol("nfs", false);
            syncRequestObject.source = getParamsByProtocol("gcp", false);
            syncRequestObject.sourceCredentials = {gcp: getCredentialsByType("gcp", selectedGCPProjectId, false)};
            break;
        }
        case "gcp-cifs": {
            syncRequestObject.source = getParamsByProtocol("gcp", false);
            syncRequestObject.sourceCredentials = {gcp: getCredentialsByType("gcp", selectedGCPProjectId, false)};
            syncRequestObject.target = getParamsByProtocol("cifs", false);
            syncRequestObject.targetCredentials = {cifs: getCredentialsByType("cifs", selectedCIFS, false, ["serverName", "version", "useACL", "windowsIP"])};
            break;
        }
        case "ibm-nfs": {
            syncRequestObject.target = getParamsByProtocol("nfs", false);
            syncRequestObject.source = getParamsByProtocol("ibm", false);
            syncRequestObject.sourceCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            break;
        }
        case "ibm-ibm": {
            syncRequestObject.source = getParamsByProtocol("ibm", false);
            syncRequestObject.sourceCredentials = {sgws: getCredentialsByType("objectStorage", selectedSG, false, ["serverName", "port"])};
            syncRequestObject.target = getParamsByProtocol("ibm", true);
            syncRequestObject.targetCredentials = {sgws: getCredentialsByType("objectStorage", selectedSGTarget, true, ["serverName", "port"])};
            break;
        }
        case "anf-anf": {
            syncRequestObject.source = getParamsByProtocol("anf", false);
            syncRequestObject.target = getParamsByProtocol("anf", true);
            break;
        }
        case "efs-anf": {
            syncRequestObject.source = getParamsByProtocol("efs", false);
            syncRequestObject.target = getParamsByProtocol("anf", true);
            break;
        }
        case "anf-azure": {
            syncRequestObject.source = getParamsByProtocol("anf", false);
            syncRequestObject.target = getParamsByProtocol("azure", false);
            const {connectionString} = getCredentialsByType("azure", selectedStorageAccountName, false);
            syncRequestObject.targetCredentials = {azure: {storageAccountConnectionString: connectionString}};
            break;
        }
        case "azure-anf": {
            syncRequestObject.source = getParamsByProtocol("azure", false);
            const {connectionString} = getCredentialsByType("azure", selectedStorageAccountName, false);
            syncRequestObject.sourceCredentials = {azure: {storageAccountConnectionString: connectionString}};
            syncRequestObject.target = getParamsByProtocol("anf", false);
            break;
        }
        default: {
            throw new Error(`sync-utils buildSyncObject wizard type ${wizardType} is not supported!!! please check`);
        }
    }
    return syncRequestObject;
};

export const getWizardSelectedParams = (props, wizardParams) => {
    const {_dataBrokers, _showExports, _showShares, _showBuckets, _showSGBuckets, _showAzureBlobContainers, _showGCPBuckets, _wizard: {wizardType}} = props;
    const {
        nfsServer, nfsServerTarget, nfsVersion, targetNfsVersion, cifsServer, cifsServerTarget, sgServer, sgServerTarget, dataBrokerId, dataBrokerIdTarget, isTargetDataBrokerInitiator,
        sgBucketPath, sgBucketPathTarget, bucketPath, storageClass, bucketPathTarget, storageClassTarget, exportPath, rootExport, exportPathTarget, rootExportTarget, gcpBucketPath, gcpProjectId,
        sharePath, sharePathTarget, rootShareTarget, rootShare, cifsVersion, targetCifsVersion, relationshipTags,
        storageAccountName, storageAccountNameTarget, azureBlobContainerPath, azureBlobContainerPathTarget, blobTier, blobTierTarget
    } = wizardParams;

    const protocols = protocolByWizardType(wizardType);

    const getSelectedParamByName = (selectedParamName) => {
        return selectedParamName ? decodeAll(_.get(props, `_wizard.selectedWizardParams.${selectedParamName}`)) : undefined;
    };

    const decodeServer = (server, selectedParamName) => {
        return server ? decodeAll(server) : getSelectedParamByName(selectedParamName);
    };

    const decodePathAndRootPath = (storagePart, server, path, rootPath, selectedPathName = null, selectedRootPathName = null) => {
        let result = {path: undefined, rootPath: undefined};
        const storagePartForServer = storagePart ? storagePart[server] : null;
        if (storagePartForServer && storagePartForServer.succeeded && storagePartForServer.data) {
            result.path = path ? decodeAll(path) : getSelectedParamByName(selectedPathName);
            result.rootPath = rootPath ? decodeAll(rootPath) : getSelectedParamByName(selectedRootPathName);
        }
        return result;
    };

    const decodeBucketAndBucketPath = (storagePart, bucketPath) => {
        const decodedBucketPath = decodeAll(bucketPath);
        const bucket = decodedBucketPath && storagePart && storagePart.succeeded && storagePart.data ? storagePart.data.find(bucket => bucket.name === decodedBucketPath.split('/')[0]) : null;
        return {bucket, bucketPath: bucket ? decodedBucketPath : null};
    };

    const getParamsByProtocol = (protocol, isTargetOfSameSource) => {
        switch (protocol) {
            case "nfs":
            case "snfs":
            case "cvs":
            case "anf":
            case "efs": {
                const decodedPathAndRootPath = isTargetOfSameSource ? decodePathAndRootPath(_showExports, nfsServerTarget, exportPathTarget, rootExportTarget, "selectedExportTarget", "selectedRootExportTarget") :
                    decodePathAndRootPath(_showExports, nfsServer, exportPath, rootExport, "selectedExport", "selectedRootExport");
                return isTargetOfSameSource ? {
                    selectedNFSTarget: decodeServer(nfsServerTarget, null),
                    selectedExportTarget: decodedPathAndRootPath.path,
                    selectedRootExportTarget: decodedPathAndRootPath.rootPath,
                    targetNfsVersion
                } : {
                    selectedNFS: decodeServer(nfsServer, "selectedNFS"),
                    selectedExport: decodedPathAndRootPath.path,
                    selectedRootExport: decodedPathAndRootPath.rootPath,
                    nfsVersion
                };
            }
            case "s3": {
                const actualBucketPath = isTargetOfSameSource ? bucketPathTarget : bucketPath;
                const decodedBucketAndBucketPath = decodeBucketAndBucketPath(_showBuckets[dataBrokerId], actualBucketPath);
                return isTargetOfSameSource ? {
                    relationshipTags: relationshipTags ? decodeAll(relationshipTags) : undefined,
                    selectedBucketPathTarget: decodedBucketAndBucketPath.bucketPath,
                    selectedStorageClass: storageClassTarget
                } : {
                    relationshipTags: relationshipTags ? decodeAll(relationshipTags) : undefined,
                    selectedBucketPath: decodedBucketAndBucketPath.bucketPath,
                    selectedStorageClass: storageClass
                }
            }
            case "cifs": {
                const decodedPathAndRootPath = isTargetOfSameSource ? decodePathAndRootPath(_showShares, cifsServerTarget, sharePathTarget, rootShareTarget) :
                    decodePathAndRootPath(_showShares, cifsServer, sharePath, rootShare);
                return isTargetOfSameSource ? {
                    selectedCIFSTarget: decodeServer(cifsServerTarget),
                    selectedShareTarget: decodedPathAndRootPath.path,
                    selectedRootShareTarget: decodedPathAndRootPath.rootPath,
                    targetCifsVersion
                } : {
                    selectedCIFS: decodeServer(cifsServer),
                    selectedShare: decodedPathAndRootPath.path,
                    selectedRootShare: decodedPathAndRootPath.rootPath,
                    cifsVersion
                }
            }
            case "sgws" :
            case "ibm" : {
                const relevantDecodedServer = isTargetOfSameSource ? decodeServer(sgServerTarget) : decodeServer(sgServer);
                const decodedBucketAndBucketPath = isTargetOfSameSource ? decodeBucketAndBucketPath(_showSGBuckets[relevantDecodedServer], sgBucketPathTarget ) : decodeBucketAndBucketPath(_showSGBuckets[relevantDecodedServer], sgBucketPath);

                return isTargetOfSameSource ? {
                    selectedSGBucketTarget: decodedBucketAndBucketPath.bucket,
                    selectedSGBucketPathTarget: decodedBucketAndBucketPath.bucketPath,
                    selectedSGTarget: relevantDecodedServer
                } : {
                    selectedSGBucket: decodedBucketAndBucketPath.bucket,
                    selectedSGBucketPath: decodedBucketAndBucketPath.bucketPath,
                    selectedSG: relevantDecodedServer
                }
            }
            case "gcp" : {
                const decodeProjectId = decodeServer(gcpProjectId);
                const decodedBucketAndBucketPath = decodeBucketAndBucketPath(_showGCPBuckets[decodeProjectId], gcpBucketPath);
                return {
                    selectedGCPBucket: decodedBucketAndBucketPath.bucket,
                    selectedGCPBucketPath: decodedBucketAndBucketPath.bucketPath,
                    selectedGCPProjectId: decodeProjectId
                }
            }
            case "azure" : {
                const actualStoragePart = isTargetOfSameSource ? _showAzureBlobContainers[storageAccountNameTarget] : _showAzureBlobContainers[storageAccountName];
                const decodedBucketAndBucketPath = isTargetOfSameSource ? decodeBucketAndBucketPath(actualStoragePart, azureBlobContainerPathTarget) : decodeBucketAndBucketPath(actualStoragePart, azureBlobContainerPath);
                return isTargetOfSameSource ? {
                    relationshipTags: relationshipTags ? decodeAll(relationshipTags) : undefined,
                    selectedStorageAccountNameTarget: decodeAll(storageAccountNameTarget),
                    selectedAzureBlobContainerPathTarget: decodedBucketAndBucketPath.bucketPath,
                    selectedBlobTier: blobTierTarget

                }:{
                    relationshipTags: relationshipTags ? decodeAll(relationshipTags) : undefined,
                    selectedStorageAccountName: decodeAll(storageAccountName),
                    selectedAzureBlobContainerPath: decodedBucketAndBucketPath.bucketPath,
                    selectedBlobTier: blobTier
                }
            }
            default: {
                return {};
            }
        }
    };

    const getSpecificSelectedWizardParamsByWizardType = (wizardType) => {
        switch (wizardType) {
            case "nfs-s3":
            case "nfs-sgws":
            case "nfs-gcp":
            case "nfs-ibm":
            case "nfs-azure":
            case "gcp-nfs":
            case "gcp-cifs":
            case "sgws-nfs":
            case "sgws-cifs":
            case "sgws-s3":
            case "ibm-nfs":
            case "ibm-cifs":
            case "efs-s3":
            case "cvs-s3":
            case "anf-s3":
            case "s3-efs":
            case "s3-nfs":
            case "s3-cifs":
            case "s3-sgws":
            case "s3-cvs":
            case "s3-anf":
            case "cifs-s3":
            case "cifs-sgws":
            case "cifs-ibm":
            case "cifs-azure":
            case "cifs-gcp":
            case "azure-nfs":
            case "azure-cifs":
            case "azure-anf":
            case "anf-azure": {
                return Object.assign({}, getParamsByProtocol(protocols.source), getParamsByProtocol(protocols.target));
            }
            case "s3-s3":
            case "cifs-cifs":
            case "efs-nfs":
            case "nfs-nfs":
            case "snfs-snfs":
            case "nfs-efs":
            case "nfs-cvs":
            case "nfs-anf":
            case "cvs-nfs":
            case "anf-nfs":
            case "ibm-ibm":
            case "sgws-sgws":
            case "azure-azure":
            case "anf-anf":
            case "efs-anf": {
                return Object.assign({}, getParamsByProtocol(protocols.source), getParamsByProtocol(protocols.target, true)); //the target is the same as the source
            }
            default: {
                throw new Error(`sync-utils getWizardSelectedParams wizard type ${wizardType} is not supported!!! please check`);
            }
        }

    };

    const selectedDataBroker = _dataBrokers && _dataBrokers.succeeded && _dataBrokers.data.length > 0 ? _dataBrokers.data.find(dataBroker => dataBroker.id === dataBrokerId) : _.get(props, "_wizard.selectedWizardParams.selectedDataBroker");
    const selectedDataBrokerTarget = _dataBrokers && _dataBrokers.succeeded && _dataBrokers.data.length > 0 ? _dataBrokers.data.find(dataBroker => dataBroker.id === dataBrokerIdTarget) : _.get(props, "_wizard.selectedWizardParams.selectedDataBrokerTarget");
    const selectedIsTargetDataBrokerInitiator = isTargetDataBrokerInitiator ? isTargetDataBrokerInitiator : "false";

    const selectedWizardParams = Object.assign({}, {
        selectedDataBroker,
        selectedDataBrokerTarget,
        selectedIsTargetDataBrokerInitiator
    }, getSpecificSelectedWizardParamsByWizardType(wizardType));

    return selectedWizardParams;
};


