import * as settings from "./../../src/utils/settings-utils";

describe("createSettingsObjectFromFormValues", () => {
    test("should build defaults object", () => {
        expect(settings.createSettingsObjectFromFormValues({
            graceValue: settings.SETTINGS_DEFAULTS.gracePeriod,
            graceResolution: {value: "Seconds"},
            gracePeriodOn: settings.EXCLUDE_FILES_OPTION,
            deleteOnTarget: settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT,
            fileTypes: [settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks],
            retries: {value: settings.SETTINGS_DEFAULTS.retries}
        })).toEqual(settings.SETTINGS_DEFAULTS);
    });
});

describe("getFormValuesFromSettings", () => {
    test("should build object from default values", () => {
        expect(settings.getFormValuesFromSettings(settings.SETTINGS_DEFAULTS)).toEqual({
            graceValue: settings.SETTINGS_DEFAULTS.gracePeriod,
            graceResolution: {label: "Seconds", value: "Seconds"},
            gracePeriodOn: settings.EXCLUDE_FILES_OPTION,
            deleteOnTarget: settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT,
            fileTypes: [settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks],
            retries: {label: `${settings.SETTINGS_DEFAULTS.retries}`, value: settings.SETTINGS_DEFAULTS.retries}
        });
    });
});

describe("getGracePeriod", () => {
    test("should return 30", () => {
        expect(settings.getGracePeriod(30, {value: "Seconds"}, true)).toEqual(30);
    });
    test("should return 0", () => {
        expect(settings.getGracePeriod(0, {value: "Seconds"}, true)).toEqual(0);
    });
    test("should return 120", () => {
        expect(settings.getGracePeriod(2, {value: "Minutes"}, true)).toEqual(120);
    });
    test("should return 18000", () => {
        expect(settings.getGracePeriod(5, {value: "Hours"}, true)).toEqual(18000);
    });
    test("should return 0", () => {
        expect(settings.getGracePeriod(30, {value: "Hours"}, false)).toEqual(0);
    });
});

describe("getFormGraceResolution", () => {
    test("should return 30 Seconds", () => {
        expect(settings.getFormGraceResolution(30)).toEqual({count: 30, resolution: {label: "Seconds", value: "Seconds"}});
    });
    test("should return 5 Hours", () => {
        expect(settings.getFormGraceResolution(18000)).toEqual({count: 5, resolution: {label: "Hours", value: "Hours"}});
    });
    test("should return 0", () => {
        expect(settings.getFormGraceResolution(0)).toEqual({count: 0, resolution: {label: "Seconds", value: "Seconds"}});
    });
});

describe("getDeleteOnTargetBool", () => {
    test("should return truth", () => {
        expect(settings.getDeleteOnTargetBool(settings.DELETE_FILES_FROM_TARGET_OPTION_TEXT)).toEqual(true);
    });
    test("should return false", () => {
        expect(settings.getDeleteOnTargetBool(settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT)).toEqual(false);
    });
});

describe("getFormDeleteOnTarget", () => {
    test("should return DELETE_FILES_FROM_TARGET_OPTION_TEXT", () => {
        expect(settings.getFormDeleteOnTarget(true)).toEqual(settings.DELETE_FILES_FROM_TARGET_OPTION_TEXT);
    });
    test("should return NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT", () => {
        expect(settings.getFormDeleteOnTarget(false)).toEqual(settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT);
    });
});

describe("getFileTypesObj", () => {
    test("should return only files", () => {
        expect(settings.getFileTypesObj([settings.fileTypesMap.files])).toEqual({files: true, directories: false, symlinks: false});
    });
    test("should return all types", () => {
        expect(settings.getFileTypesObj([settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks])).toEqual({files: true, directories: true, symlinks: true});
    });
});

describe("getFormFileTypes", () => {
    test("should return only files", () => {
        expect(settings.getFormFileTypes({files: true, directories: false, symlinks: false})).toEqual([settings.fileTypesMap.files]);
    });
    test("should return all types", () => {
        expect(settings.getFormFileTypes({files: true, directories: true, symlinks: true})).toEqual([settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks]);
    });
});

